import { useCallback, useEffect, useMemo, useRef, useState } from 'react'
import { matchPath, useLocation, useNavigate } from 'react-router-dom'
import { _navParams } from './useNavParams'

export function useNav() {
	const [ctxChanged, setCtxChanged] = useState(new Date())
	const { pathname } = useLocation()
	const navigate = useNavigate()

	const localPathname = useRef(pathname)

	const processUrl = useCallback((rawUrl: string, params = {}) => {
		let url = rawUrl
		if (url.endsWith('/*')) url = url.slice(0, -2)
		if (url.indexOf('*/') >= 0) {
			const separator = rawUrl.split('/')[1]
			url = url.replace('*', localPathname.current.split('/' + separator)[0])
		}
		url = url.replace('//', '/')
		Object.keys(params).forEach(k => (url = url.replace(':' + k, params[k])))
		return url
	}, [])

	useEffect(() => {
		localPathname.current = pathname
		setCtxChanged(new Date())
	}, [pathname])

	const goTo = useCallback(
		(url: string, urlParams: any = {}, navParams: any = {}) => {
			_navParams.current = navParams
			navigate(processUrl(url, urlParams))
		},
		[navigate, processUrl]
	)

	const replaceTo = useCallback(
		(url: string, urlParams: any = {}, navParams: any = {}) => {
			_navParams.current = navParams
			navigate(processUrl(url, urlParams), { replace: true })
		},
		[navigate, processUrl]
	)

	const goToOrReplace = useCallback(
		(path: string, params) => {
			const fullPath = window.location.href.split('#')[1]
			const alreadyThere = !!matchPath({ path }, fullPath)
			if (alreadyThere) {
				replaceTo(path, params)
			} else {
				goTo(path, params)
			}
		},
		[goTo, replaceTo]
	)

	const goBack = useCallback(() => {
		if (window.history.length > 1) navigate(-1)
	}, [navigate])

	return useMemo(
		() => ({
			ctxChanged,
			pathname: localPathname.current,
			goTo,
			replaceTo,
			goToOrReplace,
			goBack,
			processUrl,
		}),
		[ctxChanged, goBack, goTo, goToOrReplace, processUrl, replaceTo]
	)
}
export default useNav
